@@ -52,21 +52,7 @@
 #include "ss7_internal.h"
 #include "xua_asp_fsm.h"
 
-/* generate a msgb containing an IPA CCM PING message */
-struct msgb *ipa_gen_ping(void)
-{
-	/* sufficient headroom for osmo_ipa_msg_push_header() */
-	struct msgb *msg = ipa_msg_alloc(16);
-	if (!msg)
-		return NULL;
-
-	msgb_put_u8(msg, IPAC_MSGT_PING);
-	ipa_prepend_header(msg, IPAC_PROTO_IPACCESS);
-
-	return msg;
-}
-
-struct msgb *ipa_to_msg(struct xua_msg *xua)
+static struct msgb *ipa_to_msg(struct xua_msg *xua)
 {
 	struct xua_msg_part *data_ie;
 	struct m3ua_data_hdr *data_hdr;
@@ -81,59 +67,51 @@ struct msgb *ipa_to_msg(struct xua_msg *xua)
 		return NULL;
 	data_hdr = (struct m3ua_data_hdr *) data_ie->dat;
 
-	/* and even the data part still has the header prepended */
-	src = data_ie->dat + sizeof(struct m3ua_data_hdr);
-	src_len = data_ie->len - sizeof(struct m3ua_data_hdr);
-
-	if (src_len == 0) {
-		LOGP(DLSS7, LOGL_NOTICE, "Discarding Tx empty IPA msg/payload\n");
-		return NULL;
-	}
-
-	switch (data_hdr->si) {
-	case MTP_SI_SCCP:
-		/* sufficient headroom for osmo_ipa_msg_push_header() */
-		msg = ipa_msg_alloc(16);
-		if (!msg)
-			return NULL;
-		dst = msgb_put(msg, src_len);
-		memcpy(dst, src, src_len);
-		osmo_ipa_msg_push_header(msg, IPAC_PROTO_SCCP);
-		return msg;
-	case MTP_SI_NI11_OSMO_IPA:
-		/* Process our SI extension: full IPA with hdr + payload, used in SCCPlite
-		 * between BSC and MSC to send MGCP and CTRL over IPA multiplex */
-		msg = msgb_alloc(src_len, "MTP_SI_NI11_OSMO_IPA");
-		if (!msg)
-			return NULL;
-		dst = msgb_put(msg, src_len);
-		memcpy(dst, src, src_len);
-		return msg;
-	default:
+	if (data_hdr->si != MTP_SI_SCCP) {
 		LOGP(DLSS7, LOGL_ERROR, "Cannot transmit non-SCCP SI (%u) to IPA peer\n",
 		     data_hdr->si);
 		return NULL;
 	}
+
+	/* and even the data part still has the header prepended */
+	src = data_ie->dat + sizeof(struct m3ua_data_hdr);
+	src_len = data_ie->len - sizeof(struct m3ua_data_hdr);
+
+	/* sufficient headroom for osmo_ipa_msg_push_header() */
+	msg = ipa_msg_alloc(16);
+	if (!msg)
+		return NULL;
+
+	dst = msgb_put(msg, src_len);
+	memcpy(dst, src, src_len);
+
+	/* TODO: if we ever need something beyond SCCP, we can use the
+	 * M3UA SIO to determine the protocol */
+	osmo_ipa_msg_push_header(msg, IPAC_PROTO_SCCP);
+	return msg;
 }
 
 /*! \brief Send a given xUA message via a given IPA "Application Server"
  *  \param[in] as Application Server through which to send \a xua
  *  \param[in] xua xUA message to be sent
- *  \return 0 on success; negative on error
- *
- *  This function takes ownership of xua msg passed to it.
- */
+ *  \return 0 on success; negative on error */
 int ipa_tx_xua_as(struct osmo_ss7_as *as, struct xua_msg *xua)
 {
+	struct msgb *msg;
 	OSMO_ASSERT(as->cfg.proto == OSMO_SS7_ASP_PROT_IPA);
 
-	return xua_as_transmit_msg(as, xua);
+	msg = ipa_to_msg(xua);
+	if (!msg) {
+		LOGPAS(as, DLSS7, LOGL_ERROR, "Error encoding IPA Msg\n");
+		return -1;
+	}
+
+	return xua_as_transmit_msg(as, msg);
 }
 
 static int ipa_rx_msg_ccm(struct osmo_ss7_asp *asp, struct msgb *msg)
 {
 	uint8_t msg_type = msg->l2h[0];
-	int rc = 0;
 
 	LOGPASP(asp, DLSS7, LOGL_DEBUG, "%s:%s\n", __func__, msgb_hexdump(msg));
 
@@ -157,10 +135,12 @@ static int ipa_rx_msg_ccm(struct osmo_ss7_asp *asp, struct msgb *msg)
 	default:
 		LOGPASP(asp, DLSS7, LOGL_NOTICE, "Unknown CCM Message 0x%02x: %s\n",
 			msg_type, msgb_hexdump(msg));
-		rc = -1;
+		return -1;
 	}
 
-	return rc;
+	msgb_free(msg);
+
+	return 0;
 }
 
 struct osmo_ss7_as *ipa_find_as_for_asp(struct osmo_ss7_asp *asp)
@@ -180,7 +160,7 @@ struct osmo_ss7_as *ipa_find_as_for_asp(struct osmo_ss7_asp *asp)
 }
 
 /* Patch a SCCP message and add point codes to Called/Calling Party (if missing) */
-static struct msgb *patch_sccp_with_pc(const struct osmo_ss7_asp *asp, const struct msgb *sccp_msg_in,
+static struct msgb *patch_sccp_with_pc(struct osmo_ss7_asp *asp, struct msgb *sccp_msg_in,
 					uint32_t opc, uint32_t dpc)
 {
 	struct osmo_sccp_addr addr;
@@ -193,8 +173,11 @@ static struct msgb *patch_sccp_with_pc(const struct osmo_ss7_asp *asp, const str
 	if (!sua) {
 		LOGPASP(asp, DLSS7, LOGL_ERROR, "Couldn't convert SCCP to SUA: %s\n",
 			msgb_hexdump(sccp_msg_in));
+		msgb_free(sccp_msg_in);
 		return NULL;
 	}
+	/* free the input message and work with SUA version instead */
+	msgb_free(sccp_msg_in);
 
 	rc = sua_addr_parse(&addr, sua, SUA_IEI_DEST_ADDR);
 	switch (rc) {
@@ -240,25 +223,24 @@ static struct msgb *patch_sccp_with_pc(const struct osmo_ss7_asp *asp, const str
 	return sccp_msg_out;
 }
 
-/* Received an IPA frame containing either SCCP, MGCP or CTRL over the IPA multiplex.
- * All those are either forwarded (routing) or locally distributed up the stack to an MTP User. */
-static int ipa_rx_msg_up(struct osmo_ss7_asp *asp, struct msgb *msg, uint8_t sls)
+static int ipa_rx_msg_sccp(struct osmo_ss7_asp *asp, struct msgb *msg, uint8_t sls)
 {
 	int rc;
-	enum ipaccess_proto ipa_proto = osmo_ipa_msgb_cb_proto(msg);
 	struct m3ua_data_hdr data_hdr;
-	struct xua_msg *xua = NULL;
+	struct xua_msg *xua;
 	struct osmo_ss7_as *as = ipa_find_as_for_asp(asp);
 	uint32_t opc, dpc;
 
 	if (!as) {
 		LOGPASP(asp, DLSS7, LOGL_ERROR, "Rx message for IPA ASP without AS?!\n");
+		msgb_free(msg);
 		return -1;
 	}
 
 	rate_ctr_inc2(as->ctrg, SS7_AS_CTR_RX_MSU_TOTAL);
-	OSMO_ASSERT(sls <= 0xf);
-	rate_ctr_inc2(as->ctrg, SS7_AS_CTR_RX_MSU_SLS_0 + sls);
+
+	/* pull the IPA header */
+	msgb_pull_to_l2(msg);
 
 	/* We have received an IPA-encapsulated SCCP message, without
 	 * any MTP routing label.  Furthermore, the SCCP Called/Calling
@@ -271,105 +253,80 @@ static int ipa_rx_msg_up(struct osmo_ss7_asp *asp, struct msgb *msg, uint8_t sls
 	 * information inside the non-CR/CC connection oriented
 	 * messages.
 	 *
-	 * The only other alternative we have is to:
-	 *
-	 * ASP role: we assume whatever was received at the ASP/AS was meant to
-	 * reach us and hence set the DPC to the PC configured in the
-	 * 'routing-key'. In this case the 'override OPC' is used so that upper
-	 * layers can find out where it came from, so it can answer back if needed.
-	 *
-	 * SG role (STP): By default, the AS associated with the ASP assumes the
-	 * OPC of the message received was transmitted to us from the PC
-	 * configured in the 'routing-key'. If set, 'override OPC' can be used
-	 * to also tweak the originating PC, which can be useful in setups with
-	 * traffic coming from another STP where want to set eg. the OPC to the
-	 * PC of the originating AS. In this case the 'override DPC'.
-	 * allows to find out where those messages are to be routed to in the
-	 * routing decision.
-	 *
-	 * This is all quite ugly, but then what can we do :/
+	 * The only other alternative we have is to simply have a
+	 * STP (server) side configuration that specifies which point
+	 * code those messages are to be routed to, and then use this
+	 * 'override DPC' in the routing decision.  We could do the same
+	 * for the source point code to ensure responses are routed back
+	 * to us.  This is all quite ugly, but then what can we do :/
 	 */
 
 	/* First, determine the DPC and OPC to use */
-	if (asp->cfg.role == OSMO_SS7_ASP_ROLE_ASP) {
-		/* Source: Based on VTY config */
-		opc = as->cfg.pc_override.opc;
-		/* Destination: PC of the routing key */
-		dpc = as->cfg.routing_key.pc;
-	} else {
-		/* Source: if set, based on VTY config,
-		 * otherwise by default the PC of the routing key */
-		if (as->cfg.pc_override.opc_enabled)
-			opc = as->cfg.pc_override.opc;
-		else
-			opc = as->cfg.routing_key.pc;
+	if (asp->cfg.is_server) {
+		/* Source: the PC of the routing key */
+		opc = as->cfg.routing_key.pc;
 		/* Destination: Based on VTY config */
 		dpc = as->cfg.pc_override.dpc;
+	} else {
+		/* Source: Based on VTY config */
+		opc = as->cfg.pc_override.dpc;
+		/* Destination: PC of the routing key */
+		dpc = as->cfg.routing_key.pc;
 	}
 
-	/* Second, create a MTP3/M3UA label with those point codes */
+	/* Second, patch this into the SCCP message */
+	if (as->cfg.pc_override.sccp_mode == OSMO_SS7_PATCH_BOTH) {
+		msg = patch_sccp_with_pc(asp, msg, opc, dpc);
+		if (!msg) {
+			LOGPASP(asp, DLSS7, LOGL_ERROR, "Unable to patch PC into SCCP message; dropping\n");
+			return -1;
+		}
+	}
+
+	/* Third, create a MTP3/M3UA label with those point codes */
 	memset(&data_hdr, 0, sizeof(data_hdr));
+	data_hdr.si = MTP_SI_SCCP;
 	data_hdr.opc = osmo_htonl(opc);
 	data_hdr.dpc = osmo_htonl(dpc);
 	data_hdr.sls = sls;
 	data_hdr.ni = as->inst->cfg.network_indicator;
-
-	switch (ipa_proto) {
-	case IPAC_PROTO_SCCP:
-		/* Third, patch this into the SCCP message and create M3UA message in XUA structure  */
-		data_hdr.si = MTP_SI_SCCP;
-		if (as->cfg.pc_override.sccp_mode == OSMO_SS7_PATCH_BOTH) {
-			struct msgb *msg_patched = patch_sccp_with_pc(asp, msg, opc, dpc);
-			if (!msg_patched) {
-				LOGPASP(asp, DLSS7, LOGL_ERROR, "Unable to patch PC into SCCP message; dropping\n");
-				return -1;
-			}
-			xua = m3ua_xfer_from_data(&data_hdr, msgb_data(msg_patched), msgb_length(msg_patched));
-			msgb_free(msg_patched);
-		} else {
-			xua = m3ua_xfer_from_data(&data_hdr, msgb_data(msg), msgb_length(msg));
-		}
-		break;
-	default:
-		/* Submit IPA headers+payload is up the stack so MTP-USER can receive it. This is useful to
-		 * obtain MGCP/CTRL proto payloads from the SCCPLite IPA multiplex. */
-		data_hdr.si = MTP_SI_NI11_OSMO_IPA;
-		osmo_ipa_msg_push_headers(msg, ipa_proto, osmo_ipa_msgb_cb_proto_ext(msg));
-		xua = m3ua_xfer_from_data(&data_hdr, msgb_data(msg), msgb_length(msg));
-		break;
-	}
-
+	/* Create M3UA message in XUA structure */
+	xua = m3ua_xfer_from_data(&data_hdr, msgb_l2(msg), msgb_l2len(msg));
+	msgb_free(msg);
 	/* Update xua->mtp with values from data_hdr */
 	m3ua_dh_to_xfer_param(&xua->mtp, &data_hdr);
 
-	/* Pass on as if we had received it from an M3UA ASP.
-	 * xua ownership is passed here: */
+	/* Pass on as if we had received it from an M3UA ASP */
 	rc = m3ua_hmdc_rx_from_l2(asp->inst, xua);
+	xua_msg_free(xua);
 	return rc;
 }
 
 /*! \brief process M3UA message received from socket
  *  \param[in] asp Application Server Process receiving \a msg
- *  \param[in] msg received message buffer. It is kept owned by the caller.
+ *  \param[in] msg received message buffer. Callee takes ownership!
  *  \param[in] sls The SLS (signaling link selector) field to use in the generated M3UA header
  *  \returns 0 on success; negative on error */
 int ipa_rx_msg(struct osmo_ss7_asp *asp, struct msgb *msg, uint8_t sls)
 {
+	struct ipaccess_head *hh;
 	int rc;
 
 	OSMO_ASSERT(asp->cfg.proto == OSMO_SS7_ASP_PROT_IPA);
 
-	/* Here IPA headers have already been validated and were stored in
-	 * osmo_ipa_msgb_cb_proto(_ext)(), and msgb_data() and msgb_l2() both
-	 * point to IPA payload. */
+	/* osmo_ipa_process_msg() will already have verified length
+	 * consistency and set up l2h pointer */
+	hh = (struct ipaccess_head *) msg->l1h;
 
-	switch (osmo_ipa_msgb_cb_proto(msg)) {
+	switch (hh->proto) {
 	case IPAC_PROTO_IPACCESS:
 		rc = ipa_rx_msg_ccm(asp, msg);
 		break;
-	default:
-		rc = ipa_rx_msg_up(asp, msg, sls);
+	case IPAC_PROTO_SCCP:
+		rc = ipa_rx_msg_sccp(asp, msg, sls);
 		break;
+	default:
+		rc = ss7_asp_rx_unknown(asp, hh->proto, msg);
 	}
 
 	return rc;
