@@ -124,13 +124,6 @@ struct value_string osmo_ss7_asp_protocol_vals[] = {
 	{ 0, NULL }
 };
 
-const struct value_string osmo_ss7_asp_admin_state_names[] = {
-	{ OSMO_SS7_ASP_ADM_S_SHUTDOWN,	"SHUTDOWN" },
-	{ OSMO_SS7_ASP_ADM_S_BLOCKED,	"BLOCKED" },
-	{ OSMO_SS7_ASP_ADM_S_ENABLED,	"ENABLED" },
-	{ 0, NULL }
-};
-
 const struct value_string osmo_ss7_asp_role_names[] = {
 	{ OSMO_SS7_ASP_ROLE_ASP,	"ASP" },
 	{ OSMO_SS7_ASP_ROLE_SG,		"SG" },
@@ -560,8 +553,6 @@ struct osmo_ss7_asp *ss7_asp_alloc(struct osmo_ss7_instance *inst, const char *n
 	}
 	rate_ctr_group_set_name(asp->ctrg, name);
 	asp->inst = inst;
-	/* ASP in "no shutdown" state by default: */
-	asp->cfg.adm_state = OSMO_SS7_ASP_ADM_S_ENABLED;
 	ss7_asp_peer_init(&asp->cfg.remote);
 	asp->cfg.remote.port = remote_port;
 	ss7_asp_peer_init(&asp->cfg.local);
@@ -618,182 +609,114 @@ static int xua_cli_read_cb(struct osmo_stream_cli *conn, int res, struct msgb *m
 static int ipa_cli_read_cb(struct osmo_stream_cli *conn, int res, struct msgb *msg);
 static int m3ua_tcp_cli_read_cb(struct osmo_stream_cli *conn, int res, struct msgb *msg);
 static int xua_cli_connect_cb(struct osmo_stream_cli *cli);
-static int xua_cli_disconnect_cb(struct osmo_stream_cli *cli);
 static int xua_cli_close_and_reconnect(struct osmo_stream_cli *cli);
 
-
-static int ss7_asp_start_client(struct osmo_ss7_asp *asp)
-{
-	uint8_t byte;
-	int rc;
-
-	if (!asp->client)
-		asp->client = osmo_stream_cli_create(asp);
-	if (!asp->client) {
-		LOGPASP(asp, DLSS7, LOGL_ERROR, "Unable to create stream"
-			" client for ASP %s\n", asp->cfg.name);
-		return -1;
-	}
-	osmo_stream_cli_set_name(asp->client, asp->cfg.name);
-	osmo_stream_cli_set_nodelay(asp->client, true);
-	osmo_stream_cli_set_addrs(asp->client, (const char **)asp->cfg.remote.host, asp->cfg.remote.host_cnt);
-	osmo_stream_cli_set_port(asp->client, asp->cfg.remote.port);
-	osmo_stream_cli_set_local_addrs(asp->client, (const char **)asp->cfg.local.host, asp->cfg.local.host_cnt);
-	osmo_stream_cli_set_local_port(asp->client, asp->cfg.local.port);
-	osmo_stream_cli_set_proto(asp->client, asp->cfg.trans_proto);
-	osmo_stream_cli_set_reconnect_timeout(asp->client, 5);
-	osmo_stream_cli_set_connect_cb(asp->client, xua_cli_connect_cb);
-	osmo_stream_cli_set_disconnect_cb(asp->client, xua_cli_disconnect_cb);
-	switch (asp->cfg.proto) {
-	case OSMO_SS7_ASP_PROT_IPA:
-		OSMO_ASSERT(asp->cfg.trans_proto == IPPROTO_TCP);
-		osmo_stream_cli_set_read_cb2(asp->client, ipa_cli_read_cb);
-		osmo_stream_cli_set_segmentation_cb(asp->client, osmo_ipa_segmentation_cb);
-		break;
-	case OSMO_SS7_ASP_PROT_M3UA:
-		if (asp->cfg.trans_proto == IPPROTO_SCTP) {
-			osmo_stream_cli_set_read_cb2(asp->client, xua_cli_read_cb);
-			osmo_stream_cli_set_segmentation_cb(asp->client, NULL);
-		} else if (asp->cfg.trans_proto == IPPROTO_TCP) {
-			osmo_stream_cli_set_read_cb2(asp->client, m3ua_tcp_cli_read_cb);
-			osmo_stream_cli_set_segmentation_cb(asp->client, xua_tcp_segmentation_cb);
-		} else
-			OSMO_ASSERT(0);
-		break;
-	default:
-		OSMO_ASSERT(asp->cfg.trans_proto == IPPROTO_SCTP);
-		osmo_stream_cli_set_read_cb2(asp->client, xua_cli_read_cb);
-		osmo_stream_cli_set_segmentation_cb(asp->client, NULL);
-		break;
-	}
-	osmo_stream_cli_set_data(asp->client, asp);
-	byte = 1; /*AUTH is needed by ASCONF. enable, don't abort socket creation if AUTH can't be enabled */
-	osmo_stream_cli_set_param(asp->client, OSMO_STREAM_CLI_PAR_SCTP_SOCKOPT_AUTH_SUPPORTED, &byte, sizeof(byte));
-	byte = 1; /* enable, don't abort socket creation if ASCONF can't be enabled */
-	osmo_stream_cli_set_param(asp->client, OSMO_STREAM_CLI_PAR_SCTP_SOCKOPT_ASCONF_SUPPORTED, &byte, sizeof(byte));
-	if (asp->cfg.sctp_init.num_ostreams_present)
-		osmo_stream_cli_set_param(asp->client, OSMO_STREAM_CLI_PAR_SCTP_INIT_NUM_OSTREAMS,
-					&asp->cfg.sctp_init.num_ostreams_value,
-					sizeof(asp->cfg.sctp_init.num_ostreams_value));
-	if (asp->cfg.sctp_init.max_instreams_present)
-		osmo_stream_cli_set_param(asp->client, OSMO_STREAM_CLI_PAR_SCTP_INIT_MAX_INSTREAMS,
-					&asp->cfg.sctp_init.max_instreams_value,
-					sizeof(asp->cfg.sctp_init.max_instreams_value));
-	if (asp->cfg.sctp_init.max_attempts_present)
-		osmo_stream_cli_set_param(asp->client, OSMO_STREAM_CLI_PAR_SCTP_INIT_MAX_ATTEMPTS,
-					&asp->cfg.sctp_init.max_attempts_value,
-					sizeof(asp->cfg.sctp_init.max_attempts_value));
-	if (asp->cfg.sctp_init.max_init_timeo_present)
-		osmo_stream_cli_set_param(asp->client, OSMO_STREAM_CLI_PAR_SCTP_INIT_TIMEOUT,
-					&asp->cfg.sctp_init.max_init_timeo_value,
-					sizeof(asp->cfg.sctp_init.max_init_timeo_value));
-	rc = osmo_stream_cli_open(asp->client);
-	return rc;
-}
-
-/* Force stream disconnection, which should trigger disconnect_cb/close_cb() and
- * announce disconnection to upper layers. */
-int ss7_asp_disconnect_stream(struct osmo_ss7_asp *asp)
-{
-	struct osmo_stream_cli *cli;
-	struct osmo_stream_srv *srv;
-
-	/* First tear down previous state if existing: */
-	if (asp->cfg.is_server) {
-		/* We are in server mode now */
-		if (asp->client) {
-			/* if we previously were in client mode,
-			 * destroy it */
-			cli = asp->client;
-			asp->client = NULL;
-			osmo_stream_cli_destroy(cli);
-		}
-		if (asp->server) {
-			/* Make sure we close the previous stream right now: */
-			srv = asp->server;
-			asp->server = NULL;
-			osmo_stream_srv_destroy(srv);
-		}
-	} else {
-		/* We are in client mode now */
-		if (asp->server) {
-			/* if we previously were in server mode,
-			 * destroy it */
-			srv = asp->server;
-			asp->server = NULL;
-			osmo_stream_srv_destroy(srv);
-		}
-		if (asp->client) {
-			/* Make sure we close the previous stream before starting a new one: */
-			osmo_stream_cli_close(asp->client);
-		}
-	}
-	return 0;
-}
-
 int osmo_ss7_asp_restart(struct osmo_ss7_asp *asp)
 {
 	int rc;
 	char bufloc[512], bufrem[512];
-	bool destroyed = asp->dyn_allocated && asp->cfg.is_server;
+	uint8_t byte;
 
 	OSMO_ASSERT(ss7_initialized);
 	ss7_asp_peer_snprintf(bufloc, sizeof(bufloc), &asp->cfg.local);
 	ss7_asp_peer_snprintf(bufrem, sizeof(bufrem), &asp->cfg.remote);
 	LOGPASP(asp, DLSS7, LOGL_INFO, "Restarting ASP %s, r=%s<->l=%s\n",
-		asp->cfg.name, bufrem, bufloc);
+	       asp->cfg.name, bufrem, bufloc);
 
-	/* First tear down previous state if existing: */
-	ss7_asp_disconnect_stream(asp);
-
-	/* Dynamic ASPs in SCTP=server are destroyed when connection is closed. */
-	if (destroyed)
-		return 0;
-
-	/* The ASP FSM must be terminated *after* tearing down the conn, so that
-	 * DISCONNECT events go up the stack */
-	if (asp->fi) {
-		osmo_fsm_inst_term(asp->fi, OSMO_FSM_TERM_REQUEST, NULL);
-		OSMO_ASSERT(!asp->fi);
-	}
-
-	/* Apply default LM FSM for client ASP */
-	if (asp->cfg.proto != OSMO_SS7_ASP_PROT_IPA &&
-	    asp->cfg.role == OSMO_SS7_ASP_ROLE_ASP &&
-	    !asp->cfg.is_server) {
-		osmo_ss7_asp_use_default_lm(asp, LOGL_DEBUG);
-	} else {
-		osmo_ss7_asp_remove_default_lm(asp);
-	}
-
-	if ((rc = xua_asp_fsm_start(asp, asp->cfg.role, LOGL_DEBUG)) < 0)
-		return rc;
-	OSMO_ASSERT(asp->fi);
-
-	if (asp->cfg.adm_state == OSMO_SS7_ASP_ADM_S_SHUTDOWN) {
-		LOGPASP(asp, DLSS7, LOGL_NOTICE, "Skipping start for ASP in administrative state %s\n",
-			get_value_string(osmo_ss7_asp_admin_state_names, OSMO_SS7_ASP_ADM_S_SHUTDOWN));
-		return 0;
-	}
-
-	/* Now start the new stream: */
-
-	if (asp->cfg.is_server) {
-		/* We are in server mode now */
-		/* FIXME: ensure we have a SCTP server */
-		LOGPASP(asp, DLSS7, LOGL_NOTICE, "ASP Restart for server "
-			"not implemented yet!\n");
-	} else {
+	if (!asp->cfg.is_server) {
 		/* We are in client mode now */
-		rc = ss7_asp_start_client(asp);
+		if (asp->server) {
+			/* if we previously were in server mode,
+			 * destroy it */
+			osmo_stream_srv_destroy(asp->server);
+			asp->server = NULL;
+		}
+		if (!asp->client)
+			asp->client = osmo_stream_cli_create(asp);
+		if (!asp->client) {
+			LOGPASP(asp, DLSS7, LOGL_ERROR, "Unable to create stream"
+				" client for ASP %s\n", asp->cfg.name);
+			return -1;
+		}
+		osmo_stream_cli_set_name(asp->client, asp->cfg.name);
+		osmo_stream_cli_set_nodelay(asp->client, true);
+		osmo_stream_cli_set_addrs(asp->client, (const char **)asp->cfg.remote.host, asp->cfg.remote.host_cnt);
+		osmo_stream_cli_set_port(asp->client, asp->cfg.remote.port);
+		osmo_stream_cli_set_local_addrs(asp->client, (const char **)asp->cfg.local.host, asp->cfg.local.host_cnt);
+		osmo_stream_cli_set_local_port(asp->client, asp->cfg.local.port);
+		osmo_stream_cli_set_proto(asp->client, asp->cfg.trans_proto);
+		osmo_stream_cli_set_reconnect_timeout(asp->client, 5);
+		osmo_stream_cli_set_connect_cb(asp->client, xua_cli_connect_cb);
+		osmo_stream_cli_set_disconnect_cb(asp->client, xua_cli_close_and_reconnect);
+		switch (asp->cfg.proto) {
+		case OSMO_SS7_ASP_PROT_IPA:
+			OSMO_ASSERT(asp->cfg.trans_proto == IPPROTO_TCP);
+			osmo_stream_cli_set_read_cb2(asp->client, ipa_cli_read_cb);
+			osmo_stream_cli_set_segmentation_cb(asp->client, osmo_ipa_segmentation_cb);
+			break;
+		case OSMO_SS7_ASP_PROT_M3UA:
+			if (asp->cfg.trans_proto == IPPROTO_SCTP) {
+				osmo_stream_cli_set_read_cb2(asp->client, xua_cli_read_cb);
+				osmo_stream_cli_set_segmentation_cb(asp->client, NULL);
+			} else if (asp->cfg.trans_proto == IPPROTO_TCP) {
+				osmo_stream_cli_set_read_cb2(asp->client, m3ua_tcp_cli_read_cb);
+				osmo_stream_cli_set_segmentation_cb(asp->client, xua_tcp_segmentation_cb);
+			} else
+				OSMO_ASSERT(0);
+			break;
+		default:
+			OSMO_ASSERT(asp->cfg.trans_proto == IPPROTO_SCTP);
+			osmo_stream_cli_set_read_cb2(asp->client, xua_cli_read_cb);
+			osmo_stream_cli_set_segmentation_cb(asp->client, NULL);
+			break;
+		}
+		osmo_stream_cli_set_data(asp->client, asp);
+		byte = 1; /*AUTH is needed by ASCONF. enable, don't abort socket creation if AUTH can't be enabled */
+		osmo_stream_cli_set_param(asp->client, OSMO_STREAM_CLI_PAR_SCTP_SOCKOPT_AUTH_SUPPORTED, &byte, sizeof(byte));
+		byte = 1; /* enable, don't abort socket creation if ASCONF can't be enabled */
+		osmo_stream_cli_set_param(asp->client, OSMO_STREAM_CLI_PAR_SCTP_SOCKOPT_ASCONF_SUPPORTED, &byte, sizeof(byte));
+		if (asp->cfg.sctp_init.num_ostreams_present)
+			osmo_stream_cli_set_param(asp->client, OSMO_STREAM_CLI_PAR_SCTP_INIT_NUM_OSTREAMS,
+						  &asp->cfg.sctp_init.num_ostreams_value,
+						  sizeof(asp->cfg.sctp_init.num_ostreams_value));
+		if (asp->cfg.sctp_init.max_instreams_present)
+			osmo_stream_cli_set_param(asp->client, OSMO_STREAM_CLI_PAR_SCTP_INIT_MAX_INSTREAMS,
+						  &asp->cfg.sctp_init.max_instreams_value,
+						  sizeof(asp->cfg.sctp_init.max_instreams_value));
+		if (asp->cfg.sctp_init.max_attempts_present)
+			osmo_stream_cli_set_param(asp->client, OSMO_STREAM_CLI_PAR_SCTP_INIT_MAX_ATTEMPTS,
+						  &asp->cfg.sctp_init.max_attempts_value,
+						  sizeof(asp->cfg.sctp_init.max_attempts_value));
+		if (asp->cfg.sctp_init.max_init_timeo_present)
+			osmo_stream_cli_set_param(asp->client, OSMO_STREAM_CLI_PAR_SCTP_INIT_TIMEOUT,
+						  &asp->cfg.sctp_init.max_init_timeo_value,
+						  sizeof(asp->cfg.sctp_init.max_init_timeo_value));
+		rc = osmo_stream_cli_open(asp->client);
 		if (rc < 0) {
 			LOGPASP(asp, DLSS7, LOGL_ERROR, "Unable to open stream"
 				" client for ASP %s, %s ==> %s\n", asp->cfg.name, bufloc, bufrem);
 			/* we don't return error in here because osmo_stream_cli_open()
-			will continue to retry (due to timeout being explicitly set with
-			osmo_stream_cli_set_reconnect_timeout() above) to connect so the error is transient */
+			   will continue to retry (due to timeout being explicitly set with
+			   osmo_stream_cli_set_reconnect_timeout() above) to connect so the error is transient */
 		}
+	} else {
+		/* We are in server mode now */
+		if (asp->client) {
+			/* if we previously were in client mode,
+			 * destroy it */
+			osmo_stream_cli_destroy(asp->client);
+			asp->client = NULL;
+		}
+		/* FIXME: ensure we have a SCTP server */
+		LOGPASP(asp, DLSS7, LOGL_NOTICE, "ASP Restart for server "
+			"not implemented yet!\n");
 	}
+
+	/* (re)start the ASP FSM */
+	if (asp->fi)
+		osmo_fsm_inst_term(asp->fi, OSMO_FSM_TERM_REQUEST, NULL);
+	asp->fi = xua_asp_fsm_start(asp, asp->cfg.role, LOGL_DEBUG);
+
 	return 0;
 }
 
@@ -879,6 +802,14 @@ static void log_sctp_notification(struct osmo_ss7_asp *asp, const char *pfx,
 int ss7_asp_ipa_srv_conn_rx_cb(struct osmo_stream_srv *conn, int res, struct msgb *msg)
 {
 	struct osmo_ss7_asp *asp = osmo_stream_srv_get_data(conn);
+	struct osmo_stream_srv_link *link = osmo_stream_srv_get_master(conn);
+
+	/* Reparent msg to srv_link, to avoid "msg" being automatically freed if
+	 * "conn" is teared down during msg handling (or if its associated
+	 * dynamic ASP becomes unused), which would then result in a double-free
+	 * if same code path then explicitly frees the msgb through msgb_free().
+	 */
+	talloc_steal(link, msg);
 
 	if (res <= 0) {
 		if (res == -EAGAIN) {
@@ -892,17 +823,27 @@ int ss7_asp_ipa_srv_conn_rx_cb(struct osmo_stream_srv *conn, int res, struct msg
 
 	msg->dst = asp;
 	rate_ctr_inc2(asp->ctrg, SS7_ASP_CTR_PKT_RX_TOTAL);
-	return ipa_rx_msg(asp, msg, asp->ipa.sls);
+	/* we simply use the lower 4 bits of the asp_id, which is initialized to a pseudo-random value upon
+	 * connect */
+	return ipa_rx_msg(asp, msg, asp->asp_id & 0xf);
 }
 
 /* netif code tells us we can read something from the socket */
 int ss7_asp_xua_srv_conn_rx_cb(struct osmo_stream_srv *conn, int res, struct msgb *msg)
 {
 	struct osmo_ss7_asp *asp = osmo_stream_srv_get_data(conn);
+	struct osmo_stream_srv_link *link = osmo_stream_srv_get_master(conn);
 	unsigned int ppid;
 	int flags;
 	int rc = 0;
 
+	/* Reparent msg to srv_link, to avoid "msg" being automatically freed if
+	 * "conn" is teared down during msg handling (or if its associated
+	 * dynamic ASP becomes unused), which would then result in a double-free
+	 * if same code path then explicitly frees the msgb through msgb_free().
+	 */
+	talloc_steal(link, msg);
+
 	/* process the received xUA message */
 	flags = msgb_sctp_msg_flags(msg);
 
@@ -964,9 +905,17 @@ int xua_tcp_segmentation_cb(struct msgb *msg)
 int ss7_asp_m3ua_tcp_srv_conn_rx_cb(struct osmo_stream_srv *conn, int res, struct msgb *msg)
 {
 	struct osmo_ss7_asp *asp = osmo_stream_srv_get_data(conn);
+	struct osmo_stream_srv_link *link = osmo_stream_srv_get_master(conn);
 	const struct xua_common_hdr *hdr;
 	int rc;
 
+	/* Reparent msg to srv_link, to avoid "msg" being automatically freed if
+	 * "conn" is teared down during msg handling (or if its associated
+	 * dynamic ASP becomes unused), which would then result in a double-free
+	 * if same code path then explicitly frees the msgb through msgb_free().
+	 */
+	talloc_steal(link, msg);
+
 	if (res <= 0) {
 		if (res == -EAGAIN) {
 			msgb_free(msg);
@@ -1018,6 +967,12 @@ static int xua_cli_connect_cb(struct osmo_stream_cli *cli)
 	if (asp->cfg.trans_proto == IPPROTO_SCTP) {
 		rc = ss7_asp_apply_peer_primary_address(asp);
 		rc = ss7_asp_apply_primary_address(asp);
+	} else {
+		if (asp->cfg.proto == OSMO_SS7_ASP_PROT_IPA) {
+			/* we use the lower 4 bits of the asp_id field as SLS;
+			 * let's initialize it here from a pseudo-random value */
+			asp->asp_id = rand() & 0xf;
+		}
 	}
 
 	if (asp->lm && asp->lm->prim_cb) {
@@ -1044,25 +999,15 @@ static void xua_cli_close(struct osmo_stream_cli *cli)
 
 static int xua_cli_close_and_reconnect(struct osmo_stream_cli *cli)
 {
-	struct osmo_ss7_asp *asp = osmo_stream_cli_get_data(cli);
-	LOGPASP(asp, DLSS7, LOGL_NOTICE, "Closing and reconnecting ASP\n");
 	xua_cli_close(cli);
-	if (asp->cfg.adm_state != OSMO_SS7_ASP_ADM_S_SHUTDOWN)
-		osmo_stream_cli_reconnect(cli);
-	return 0;
-}
-
-static int xua_cli_disconnect_cb(struct osmo_stream_cli *cli)
-{
-	struct osmo_ss7_asp *asp = osmo_stream_cli_get_data(cli);
-	LOGPASP(asp, DLSS7, LOGL_NOTICE, "disconnect_cb() from lower layers, reconnecting\n");
-	xua_cli_close_and_reconnect(cli);
+	osmo_stream_cli_reconnect(cli);
 	return 0;
 }
 
 /* read call-back for IPA/SCCPlite socket */
 static int ipa_cli_read_cb(struct osmo_stream_cli *conn, int res, struct msgb *msg)
 {
+	int fd = osmo_stream_cli_get_fd(conn);
 	struct osmo_ss7_asp *asp = osmo_stream_cli_get_data(conn);
 
 	if (res <= 0) {
@@ -1077,7 +1022,9 @@ static int ipa_cli_read_cb(struct osmo_stream_cli *conn, int res, struct msgb *m
 
 	msg->dst = asp;
 	rate_ctr_inc2(asp->ctrg, SS7_ASP_CTR_PKT_RX_TOTAL);
-	return ipa_rx_msg(asp, msg, asp->ipa.sls);
+	/* we can use the 'fd' return value of osmo_stream_srv_get_fd() here unverified as all we do
+	 * is 'roll the dice' to obtain a 4-bit SLS value. */
+	return ipa_rx_msg(asp, msg, fd & 0xf);
 }
 
 /* read call-back for M3UA-over-TCP socket */
@@ -1205,7 +1152,6 @@ int osmo_ss7_asp_send(struct osmo_ss7_asp *asp, struct msgb *msg)
 	}
 
 	rate_ctr_inc2(asp->ctrg, SS7_ASP_CTR_PKT_TX_TOTAL);
-	LOGPASP(asp, DLSS7, LOGL_DEBUG, "Tx %d bytes: %s\n", msg->len, msgb_hexdump(msg));
 
 	if (asp->cfg.is_server) {
 		if (!asp->server) {
@@ -1325,52 +1271,3 @@ int ss7_asp_get_fd(const struct osmo_ss7_asp *asp)
 	}
 	return -1;
 }
-
-/* Apply sane configs for unconfigured options and restart the ASP.  */
-void ss7_asp_restart_after_reconfigure(struct osmo_ss7_asp *asp)
-{
-	/* Make sure proper defaults values are applied if user didn't provide
-	* specific default values */
-	ss7_asp_set_default_peer_hosts(asp);
-
-	osmo_ss7_asp_restart(asp);
-}
-
-/* Obtain all routing contexts (in network byte order) that exist within the given ASP */
-static unsigned int _ss7_asp_get_all_rctx(const struct osmo_ss7_asp *asp, uint32_t *rctx, unsigned int rctx_size,
-					  const struct osmo_ss7_as *excl_as, bool network_byte_order)
-{
-	unsigned int count = 0;
-	struct osmo_ss7_as *as;
-
-	llist_for_each_entry(as, &asp->inst->as_list, list) {
-		if (as == excl_as)
-			continue;
-		if (!osmo_ss7_as_has_asp(as, asp))
-			continue;
-		if (as->cfg.routing_key.context == 0)
-			continue;
-		if (count >= rctx_size)
-			break;
-		if (network_byte_order)
-			rctx[count] = htonl(as->cfg.routing_key.context);
-		else
-			rctx[count] = as->cfg.routing_key.context;
-		count++;
-	}
-	return count;
-}
-
-/* Obtain all routing contexts (in network byte order) that exist within the given ASP */
-unsigned int ss7_asp_get_all_rctx_be(const struct osmo_ss7_asp *asp, uint32_t *rctx, unsigned int rctx_size,
-				     const struct osmo_ss7_as *excl_as)
-{
-	return _ss7_asp_get_all_rctx(asp, rctx, rctx_size, excl_as, true);
-}
-
-/* Obtain all routing contexts (in host byte order) that exist within the given ASP */
-unsigned int ss7_asp_get_all_rctx(const struct osmo_ss7_asp *asp, uint32_t *rctx, unsigned int rctx_size,
-				     const struct osmo_ss7_as *excl_as)
-{
-	return _ss7_asp_get_all_rctx(asp, rctx, rctx_size, excl_as, false);
-}
