@@ -43,15 +43,9 @@ struct osmo_ss7_asp {
 	/*! pre-formatted human readable local/remote socket name */
 	char *sock_name;
 
-	/* ASP Identifier for ASP-UP + NTFY, as received by the peer.
-	 * (In IPA ASPs it's used internally to hold 4-bit SLS).
-	 * FIXME: This should actually be stored in a AS-ASP relation, since it
-	 *        can be different per AS, see RFC4666 3.5.1
-	 * "The optional ASP Identifier parameter contains a unique value that
-	 *  is locally significant among the ASPs that support an AS".
-	 */
-	uint32_t remote_asp_id;
-	bool remote_asp_id_present;
+	/* ASP Identifier for ASP-UP + NTFY */
+	uint32_t asp_id;
+	bool asp_id_present;
 
 	/* Layer Manager to which we talk */
 	const struct osmo_xua_layer_manager *lm;
@@ -69,15 +63,6 @@ struct osmo_ss7_asp {
 	/*! Pending message for non-blocking IPA read */
 	struct msgb *pending_msg;
 
-	/* IPA proto ASP specific fields. */
-	struct {
-		/* Incoming IPA PDUs have no SLS field, hence a potentially
-		 * unique one within AS is assigned to this ASP and applied
-		 * manually when received. */
-		uint8_t sls:4;
-		bool sls_assigned;
-	} ipa;
-
 	struct {
 		char *name;
 		char *description;
@@ -118,18 +103,11 @@ struct osmo_ss7_asp *ss7_asp_alloc(struct osmo_ss7_instance *inst, const char *n
 bool ss7_asp_set_default_peer_hosts(struct osmo_ss7_asp *asp);
 bool ss7_asp_is_started(const struct osmo_ss7_asp *asp);
 int ss7_asp_get_fd(const struct osmo_ss7_asp *asp);
-int ss7_asp_disconnect_stream(struct osmo_ss7_asp *asp);
 
 int ss7_asp_apply_peer_primary_address(const struct osmo_ss7_asp *asp);
 int ss7_asp_apply_primary_address(const struct osmo_ss7_asp *asp);
 int ss7_asp_apply_new_local_address(const struct osmo_ss7_asp *asp, unsigned int loc_idx);
 int ss7_asp_apply_drop_local_address(const struct osmo_ss7_asp *asp, unsigned int loc_idx);
 
-void ss7_asp_restart_after_reconfigure(struct osmo_ss7_asp *asp);
-void osmo_ss7_asp_remove_default_lm(struct osmo_ss7_asp *asp);
-
-unsigned int ss7_asp_get_all_rctx_be(const struct osmo_ss7_asp *asp, uint32_t *rctx, unsigned int rctx_size,
-				  const struct osmo_ss7_as *excl_as);
-
 #define LOGPASP(asp, subsys, level, fmt, args ...) \
-	_LOGSS7((asp)->inst, subsys, level, "ASP(%s) " fmt, (asp)->cfg.name, ## args)
+	_LOGSS7((asp)->inst, subsys, level, "asp-%s: " fmt, (asp)->cfg.name, ## args)
